package com.qs.mp.pay.service.impl;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;
import com.huifu.bspay.sdk.opps.client.BasePayClient;
import com.huifu.bspay.sdk.opps.core.BasePay;
import com.huifu.bspay.sdk.opps.core.config.MerConfig;
import com.huifu.bspay.sdk.opps.core.request.*;
import com.huifu.bspay.sdk.opps.core.utils.DateTools;
import com.huifu.bspay.sdk.opps.core.utils.SequenceTools;
import com.qs.mp.common.constant.Constants;
import com.qs.mp.common.utils.KeyGenUtil;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.pay.domain.DougongPayRequestParam;
import com.qs.mp.pay.domain.PaymentConfig;
import com.qs.mp.pay.domain.PaymentConfigExample;
import com.qs.mp.pay.mapper.PaymentConfigMapper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import com.qs.mp.common.exception.ServiceException;

/**
 * 商家的微信支付
 */
@Component
public class DougongPayApi {
    private static final Logger log = LoggerFactory.getLogger(DougongPayApi.class);
    private static final String onlineEnvironment = "dev,prd,ali";
    @Value("${pay.dgPay.dougongMiniNoticeUrl}")
    private String dougongMiniNoticeUrl;
    @Value("${pay.dgPay.dougongMiniRefundNoticeUrl}")
    private String dougongMiniRefundNoticeUrl;
    @Value("${pay.dgPay.dougongMerchantSubmitsBackPageUrl}")
    private String dougongMerchantSubmitsBackPageUrl;
    @Value("${pay.dgPay.dougongMerchantSubmitsAsyncReceiveUrl}")
    private String dougongMerchantSubmitsAsyncReceiveUrl;
    private static final String DOUGONG_SYS_ID_FENGYI = "6666000137086944";

    @Resource
    private PaymentConfigMapper paymentConfigMapper;

    public List<PaymentConfig> getAvailablePaymentConfigList(){
        PaymentConfigExample paymentConfigExample = new PaymentConfigExample();
        PaymentConfigExample.Criteria criteria = paymentConfigExample.createCriteria();
        criteria.andDelTagEqualTo(0);
        criteria.andAvailableTagEqualTo(1);
        return this.paymentConfigMapper.selectByExample(paymentConfigExample);
    }

    public Map<String, String> unifiedOrderDougong(DougongPayRequestParam dougongPayRequestParam, String tradeType, PaymentConfig paymentConfig) throws ServiceException{
        //检测必填参数
        if (StringUtils.isEmpty(dougongPayRequestParam.getTradesOrderId())) {
            throw new ServiceException("缺少统一支付接口必填参数tradesOrderId！");
        } else if (Objects.isNull(dougongPayRequestParam.getTotalFee())) {
            throw new ServiceException("缺少统一支付接口必填参数total_fee！");
        }

        //关联参数
        if ("JSAPI".equals(dougongPayRequestParam.getTradeType()) && (StringUtils.isEmpty(dougongPayRequestParam.getOpenid()) && StringUtils.isEmpty(dougongPayRequestParam.getSubOpenid()))) {
            throw new ServiceException("统一支付接口中，缺少必填参数openid！trade_type为JSAPI时，openid为必填参数！");
        }
        if ("NATIVE".equals(dougongPayRequestParam.getTradeType()) && StringUtils.isEmpty(dougongPayRequestParam.getProductId())) {
            throw new ServiceException("统一支付接口中，缺少必填参数product_id！trade_type为NATIVE时，product_id为必填参数！");
        }
        String orderFee = BigDecimal.valueOf(dougongPayRequestParam.getTotalFee()).divide(new BigDecimal("100"), 2, RoundingMode.HALF_UP).toPlainString();
        log.info("unifiedOrderDougong--orderFee::{},totalFee:{}",orderFee,dougongPayRequestParam.getTotalFee());
        /**
         * T_JSAPI: 微信公众号
         * T_MINIAPP: 微信小程序
         * A_JSAPI: 支付宝JS
         * A_NATIVE: 支付宝正扫
         * U_NATIVE: 银联正扫
         * U_JSAPI: 银联JS
         * D_NATIVE: 数字人民币正扫
         */
        log.info("dougong pay jspay api request:{}",dougongPayRequestParam.getTradesOrderId());
//        List<PaymentConfig> paymentConfigList = getAvailablePaymentConfigList();
//        PaymentConfig paymentConfig = CollectionUtil.isNotEmpty(paymentConfigList) ? RandomUtil.randomEle(paymentConfigList) : null;
        long huifupayStart = System.currentTimeMillis();
        String sysId = Objects.nonNull(paymentConfig) ? paymentConfig.getSysId() : "";
        String huifuId = Objects.nonNull(paymentConfig) ? paymentConfig.getHuifuId() : "";
        log.info("unifiedOrderDougong--sysId:{},huifuId:{}",sysId,huifuId);
        BasePay.debug = false;
        BasePay.prodMode = BasePay.MODE_PROD;
        MerConfig merConfig = new MerConfig();
        log.info("斗拱配置参数3：-------------------------------》" + JSONUtil.toJsonStr(paymentConfig));
        merConfig.setProcutId(Objects.nonNull(paymentConfig) ? paymentConfig.getDougongProductId() : "");
        merConfig.setSysId(sysId);
        merConfig.setRsaPrivateKey(Objects.nonNull(paymentConfig) ? paymentConfig.getPrivateKeyDougong() : "");
        merConfig.setRsaPublicKey(Objects.nonNull(paymentConfig) ? paymentConfig.getPublicKeyHuifu() : "");
        log.info("斗拱支付参数：-------------------------------》"+JSONUtil.toJsonStr(merConfig));
        Map<String, Object> result = null;
        try{
            log.info("Dougong-pay-initWithMerConfig0");
            BasePay.initWithMerConfig(merConfig);
            log.info("Dougong-pay-initWithMerConfig1");
            V2TradePaymentJspayRequest request = new V2TradePaymentJspayRequest();
            request.setReqDate(DateTools.getCurrentDateYYYYMMDD());
            request.setReqSeqId(SequenceTools.getReqSeqId32());
            request.setHuifuId(huifuId);
            request.setTradeType(tradeType);
            log.info("unifiedOrderDougong--transAmt::{}",orderFee);
            request.setTransAmt(orderFee);
            JSONObject wxData = new JSONObject();
            JSONObject aliPayData = new JSONObject();

            wxData.put("openid",dougongPayRequestParam.getOpenid());
            wxData.put("sub_openid",dougongPayRequestParam.getSubOpenid());
//            wxData.put("sub_appid",wxRequestParam.getSubAppid());
            wxData.put("sub_appid",dougongPayRequestParam.getAppid());

            log.info("ali-buyer_id::{}",dougongPayRequestParam.getOpenid());
            aliPayData.put("buyer_id",dougongPayRequestParam.getOpenid());

            request.setGoodsDesc("ql-goods-" + KeyGenUtil.newKey());
            Map<String, Object> extendInfoMap = Maps.newHashMap();

            log.info("unifiedOrderDougong--notifyUrl::{},payStyle::{}",dougongMiniNoticeUrl,dougongPayRequestParam.getPayStyle());
            log.info("回调地址---------------》{}",dougongMiniNoticeUrl);
            extendInfoMap.put("notify_url",this.dougongMiniNoticeUrl);
            if(Constants.DOUGONG_TRADE_TYPE_MINIAPP.equals(tradeType)){
                extendInfoMap.put("wx_data",wxData.toString());
            }else if(Constants.DOUGONG_TRADE_TYPE_ALINATIVE.equals(tradeType)){
                extendInfoMap.put("alipay_data",aliPayData.toString());
            }

            extendInfoMap.put("remark","--");
            extendInfoMap.put("goods_desc","门店消费");
            request.setExtendInfo(extendInfoMap);
            request.setReqSeqId(dougongPayRequestParam.getHuifuPayId());

            log.info("Dougong-pay-request-params::{}",JSONUtil.toJsonStr(request));
            result = BasePayClient.request(request,false);
            log.info("微信小程序-斗拱扫台牌交易返回报文:{},请求耗时:{},tradesId:{},payStyle:{}",result,(System.currentTimeMillis() - huifupayStart),dougongPayRequestParam.getTradesOrderId(),dougongPayRequestParam.getPayStyle());
        }catch (Exception e){
            e.printStackTrace();
        }

        log.info("微信小程序-斗拱扫台牌交易返回报文1:{}",result);
        Map<String, String> payInfo = Maps.newHashMap();

        if(Constants.DOUGONG_TRADE_TYPE_MINIAPP.equals(tradeType)){
            payInfo.put("payInfo", result.get("pay_info").toString());
            payInfo.put("respCode",result.get("resp_code").toString());
            log.info("dougong pay info-wx:{}", payInfo);
        }else if(Constants.DOUGONG_TRADE_TYPE_ALINATIVE.equals(tradeType)){
            payInfo.put("payInfo", result.get("qr_code").toString());
            payInfo.put("respCode",result.get("resp_code").toString());
            log.info("dougong pay info-ali:{}", payInfo);
        }

        return payInfo;
    }


}
