package com.qs.mp.pay.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Maps;
import com.qs.mp.common.constant.SysCode;
import com.qs.mp.common.enums.PayOrderStatusEnum;
import com.qs.mp.common.utils.http.OkHttpUtil;
import com.qs.mp.pay.domain.PayOrder;
import com.qs.mp.pay.mapper.PayOrderMapper;
import com.qs.mp.pay.service.IPayOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 支付订单表 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-04
 */
@Service
public class PayOrderServiceImpl extends ServiceImpl<PayOrderMapper, PayOrder> implements IPayOrderService {

    Logger log = LoggerFactory.getLogger(PayOrderServiceImpl.class);

    @Autowired
    private IPayOrderService payOrderService;

    @Transactional
    public Map<String,Object> dougongPaySuccessCallBack(PayOrder payOrder) {
        Map<String,Object> result = SysCode.fail();
        log.info("dougongPaySuccessCallBack---------->tradesId:{},huifuPayId:{}",payOrder.getOrderId(),payOrder.getOrderNo());
        log.info("dougongPaySuccessCallBack---------->tradesOrder:{}", JSONUtil.toJsonStr(payOrder));
        try {
            if ("2".equals(payOrder.getOrderStatus())) {
                log.info("dougongPaySuccessCallBack---------->0,huifuPaId:{}", payOrder.getOrderNo());
                payOrder.setOrderStatus(PayOrderStatusEnum.SUCCESS.getValue());
                payOrderService.saveOrUpdate(payOrder);

                Map<String,String> resultx = SysCode.success();

                resultx.put(SysCode.DATA_KEY, JSONUtil.toJsonStr(payOrder));
                resultx.put(SysCode.MSG_KEY,"订单支付成功，单号:" + payOrder.getOrderId());
                log.info("dougongPaySuccessCallBack----->1");

                String xurl = "https://openapi.donghuid.com/yinLianPay/yinLianPayNotify";
                for (int i = 0; i < 5; i++) {
                    String xresult = OkHttpUtil.post(xurl, resultx);
                    log.info("向第三方继续回调结果:{}", xresult);
                    Map<String, Object> xmap = JSONUtil.toBean(xresult, Map.class);
                    if (xmap.containsKey("code")) {
                        String v = xmap.get("code").toString();
                        if (String.valueOf(SysCode.OPER_SUCCESS).equals(v)) {//1W成功，2W失败
                            break;
                        }
                    }
                }


                result = SysCode.success("dg支付回调成功,单号:{}" + payOrder.getOrderId());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("线下订单回调处理出错{}", e.getMessage());
            result = SysCode.fail("dg支付回调失败，单号:{}" + payOrder.getOrderId());
        }


        return result;
    }
}
